/***************************************************************************
 *   Copyright (C) 2006 by George Kiagiadakis   *
 *   gkiagia@users.sourceforge.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "version.h"

#include <QCoreApplication>
#include <QApplication>
#include <QString>
#include <QMessageBox>
#include <QProcess>
#include <QStringList>
#include <QRegExp>
#include "manager.h"
#include "launcher.h"
#include "reginstaller.h"
#include "winelauncher.h"
#include "confighandler.h"
#include "optionsdialog.h"
#include "macros.h"

QSettings *QtWine::settings = NULL;
QString convertFilenameToWindows( QString unixName, QString cfgName );

QString convertFilenameToWindows( QString unixName, QString cfgName )
{
	QProcess winepath;
	winepath.setEnvironment(QProcess::systemEnvironment() << QString("WINEPREFIX=%1").arg( cfgHandler->getCfgPath(cfgName) ) );

	winepath.start("winepath", QStringList() << "-w" << unixName);
	winepath.waitForFinished();
	QString windowsPath = winepath.readAllStandardOutput();
	windowsPath = windowsPath.remove('\n');

	return windowsPath;
}

int main( int argc, char **argv )
{
	int i;  bool silent=FALSE, useGUI=TRUE, ask=FALSE;
	QString logFile,configuration,winAppName,workdir;   QStringList args;

	QApplication a(argc, argv); // this is here to parse Qt options
	for(i=1; i<argc; i++) args << argv[i]; // fill the QStringList args with the contents of argv[] (without the Qt options)

	// GET COMMAND LINE OPTIONS
	if ( args.indexOf(QRegExp("^(--help|-h)$")) != -1 ) { QtWineInformation::console_help(); return 0; }
	if ( args.indexOf(QRegExp("^(--options|-o)$")) != -1 ) // launch optionsDialog and exit when finished
	{
		QtWine::settings = new QSettings(); // must be initialized. ConfigHandler must NOT be initialized
		optionsDialog od;
		od.show();
		return a.exec();
	}
	if ( (i=args.indexOf(QRegExp("^(--log)$"))) != -1)
	{
		logFile = args.takeAt(i+1); // get log file's name and remove it from the list
		args.removeAt(i); // remove --log
	}
	if ( (i=args.indexOf(QRegExp("^(--launch|-l)$"))) != -1 )
	{
		configuration = args.takeAt(i+1); // get config name and remove it from the list
		args.removeAt(i); // remove --launch or -l
		useGUI=FALSE; // start command-line (console) mode

		if ( (i=args.indexOf(QRegExp("^(--silent|-s)$"))) != -1 ) // check if console mode should be silent
		{
			silent=TRUE;
			args.removeAt(i); // remove --silent or -s
		}

		if ( (i=args.indexOf(QRegExp("^(--ask|-a)$"))) != -1 )
		{
			ask=TRUE;
			args.removeAt(i);
		}
	}
	if ( (i=args.indexOf(QRegExp("^(--workdir|-w)$"))) != -1) // TODO add to help
	{
		workdir = args.takeAt(i+1);
		args.removeAt(i);
	}

	if ( args.size() > 0 )
		winAppName = args.takeAt(0); // the first QString remaining in args (if anything remains) is the file name of the .exe or .reg file opened and the rest is the arguments passed to it.

	// Set identity for QSettings
	a.setOrganizationName("QtWine project");
	a.setOrganizationDomain("qtwine.sourceforge.net");
	a.setApplicationName("qtwine");

	// Start services
	QtWine::settings = new QSettings();
	new ConfigHandler(0);

	// START THE APPLICATION
	if ( useGUI )
	{
		if ( !winAppName.isEmpty() )
		{
			if ( winAppName.endsWith(".reg", Qt::CaseInsensitive) )
			{
					regInstaller r;
					r.setFileName( winAppName );
					r.setLog( logFile );
					r.show();
					return a.exec();
			}
			else if ( winAppName.endsWith(".exe", Qt::CaseInsensitive) or winAppName.endsWith(".exe.so", Qt::CaseInsensitive) )
			{
					launcher l;
					l.setFileName( winAppName );
					int argsCount = args.size();
					for (i=0 ; i < argsCount ; i++)
					{
						args[i].prepend('\"');
						args[i].append('\"');
					}
					l.setArguments( args.join(" ") );
					l.setLog( logFile );
					l.show();
					return a.exec();
			}
		}
		else
		{
			manager m;
			m.show();
			return a.exec();
		}
	}
	else // console mode
	{
		// convert unix filenames to windows filenames
		QFile f;
		int argsCount = args.size();
		for (i=0 ; i < argsCount ; i++)
		{
			f.setFileName( args[i] );
			if ( f.exists() )
				args.replace( i, convertFilenameToWindows(args[i], configuration) );
		}

		WineLauncher wine;
		wine.setCfgName(configuration);
		wine.setSilent(silent);
		wine.setLogFile(logFile);
		wine.setWorkDir(workdir);
		if (ask)
			wine.setQuitMode(WineLauncher::ask);
		else
			wine.setQuitMode(WineLauncher::quit);
		wine.startWine(winAppName, args);
		return a.exec();
	}
}

